@@ -8,6 +8,8 @@
 
 import os
 import sys
+import numpy as np
+from distutils.version import LooseVersion
 try:
     import cPickle as pickle
 except ImportError:
@@ -16,7 +18,13 @@ except ImportError:
 from blosc import blosc_extension as _ext
 import blosc
 
-if sys.version_info[0] < 3:
+# version number hack
+vi = sys.version_info
+PY26 = vi[0] == 2 and vi[1] == 6
+PY27 = vi[0] == 2 and vi[1] == 7
+PY3X = vi[0] == 3
+
+if vi[0] < 3:
     int_ = (int, long)
 else:
     int_ = (int,)
@@ -110,12 +118,55 @@ def set_blocksize(blocksize):
     --------
 
     >>> blosc.set_blocksize(512)
+    >>> blosc.set_blocksize(0)
 
     """
 
     _ext.set_blocksize(blocksize)
 
 
+def get_blocksize():
+    """
+    get_blocksize()
+
+    Get the blocksize currently used.
+
+    Examples
+    --------
+
+    >>> blosc.set_blocksize(0)
+    >>> blosc.get_blocksize()
+    0
+
+    """
+    return _ext.get_blocksize()
+
+
+def set_releasegil(gilstate):
+    """
+    set_releasegil( gitstate )
+
+    Sets a boolean on whether to release the Python global inter-lock (GIL)
+    during c-blosc compress and decompress operations or not.  This defaults
+    to False.
+
+    Notes
+    -----
+
+    Designed to be used with larger chunk sizes and a ThreadPool.  There is a
+    small performance penalty with releasing the GIL that will more harshly
+    penalize small block sizes.
+
+    Examples
+    --------
+
+    >>> oldReleaseState = blosc.set_releasegil(True)
+
+    """
+    gilstate = bool(gilstate)
+    return _ext.set_releasegil(gilstate)
+
+
 def compressor_list():
     """
     compressor_list()
@@ -212,6 +263,35 @@ def get_clib(bytesobj):
     return _ext.get_clib(bytesobj)
 
 
+def get_cbuffer_sizes(bytesobj):
+    """
+    get_cbuffer_sizes(bytesobj)
+
+    Return information about a compressed buffer:
+    (number of uncompressed bytes, number of compressed bytes, blocksize).
+
+    Parameters
+    ----------
+    bytesobj : str / bytes
+        The compressed buffer.
+
+    Returns
+    -------
+    out : tuple
+        The associated uncompressed bytes, compressed bytes and blocksize
+
+    Examples
+    --------
+
+    >>> s = b'0123456789'
+    >>> c = blosc.compress(s, typesize=1)
+    >>> blosc.get_cbuffer_sizes(c)
+    (10, 26, 10)
+    """
+
+    return _ext.get_cbuffer_sizes(bytesobj)
+
+
 def free_resources():
     """
     free_resources()
@@ -237,6 +317,16 @@ def free_resources():
     _ext.free_resources()
 
 
+def _check_shuffle(shuffle):
+    if shuffle not in [blosc.NOSHUFFLE, blosc.SHUFFLE, blosc.BITSHUFFLE]:
+        raise ValueError("shuffle can only be one of NOSHUFFLE, SHUFFLE"
+                         " and BITSHUFFLE.")
+    if (shuffle == blosc.BITSHUFFLE and
+        LooseVersion(blosc.blosclib_version) < LooseVersion("1.8.0")):
+        raise ValueError("You need C-Blosc 1.8.0 or higher for using"
+                         " BITSHUFFLE.")
+
+
 def _check_clevel(clevel):
     if not 0 <= clevel <= 9:
         raise ValueError("clevel can only be in the 0-9 range.")
@@ -292,10 +382,10 @@ def compress(bytesobj, typesize=8, clevel=9, shuffle=blosc.SHUFFLE,
         blosc.NOSHUFFLE, blosc.SHUFFLE and blosc.BITSHUFFLE.  The
         default is blosc.SHUFFLE.
     cname : string (optional)
-        The name of the compressor used internally in Blosc.
-        It can be any of the supported by Blosc ('blosclz',
-        'lz4', 'lz4hc', 'snappy', 'zlib' and maybe others too).
-        The default is 'blosclz'.
+        The name of the compressor used internally in Blosc. It can be
+        any of the supported by Blosc ('blosclz', 'lz4', 'lz4hc',
+        'snappy', 'zlib', 'zstd' and maybe others too). The default is
+        'blosclz'.
 
     Returns
     -------
@@ -326,6 +416,7 @@ def compress(bytesobj, typesize=8, clevel=9, shuffle=blosc.SHUFFLE,
 
     _check_input_length('bytesobj', len(bytesobj))
     _check_typesize(typesize)
+    _check_shuffle(shuffle)
     _check_clevel(clevel)
     _check_cname(cname)
 
@@ -354,10 +445,10 @@ def compress_ptr(address, items, typesize=8, clevel=9, shuffle=blosc.SHUFFLE,
         blosc.NOSHUFFLE, blosc.SHUFFLE and blosc.BITSHUFFLE.  The
         default is blosc.SHUFFLE.
     cname : string (optional)
-        The name of the compressor used internally in Blosc.
-        It can be any of the supported by Blosc ('blosclz',
-        'lz4', 'lz4hc', 'snappy', 'zlib' and maybe others too).
-        The default is 'blosclz'.
+        The name of the compressor used internally in Blosc. It can be
+        any of the supported by Blosc ('blosclz', 'lz4', 'lz4hc',
+        'snappy', 'zlib', 'zstd' and maybe others too). The default is
+        'blosclz'.
 
     Returns
     -------
@@ -418,6 +509,7 @@ def compress_ptr(address, items, typesize=8, clevel=9, shuffle=blosc.SHUFFLE,
     length = items * typesize
     _check_input_length('length', length)
     _check_typesize(typesize)
+    _check_shuffle(shuffle)
     _check_clevel(clevel)
     _check_cname(cname)
 
@@ -562,10 +654,10 @@ def pack_array(array, clevel=9, shuffle=blosc.SHUFFLE, cname='blosclz'):
         blosc.NOSHUFFLE, blosc.SHUFFLE and blosc.BITSHUFFLE.  The
         default is blosc.SHUFFLE.
     cname : string (optional)
-        The name of the compressor used internally in Blosc.
-        It can be any of the supported by Blosc ('blosclz',
-        'lz4', 'lz4hc', 'snappy', 'zlib' and maybe others too).
-        The default is 'blosclz'.
+        The name of the compressor used internally in Blosc. It can be
+        any of the supported by Blosc ('blosclz', 'lz4', 'lz4hc',
+        'snappy', 'zlib', 'zstd' and maybe others too). The default is
+        'blosclz'.
 
     Returns
     -------
@@ -601,6 +693,7 @@ def pack_array(array, clevel=9, shuffle=blosc.SHUFFLE, cname='blosclz'):
     itemsize = array.itemsize
     _check_input_length('array size', array.size*itemsize)
     _check_typesize(array.itemsize)
+    _check_shuffle(shuffle)
     _check_clevel(clevel)
     _check_cname(cname)
 
@@ -612,7 +705,7 @@ def pack_array(array, clevel=9, shuffle=blosc.SHUFFLE, cname='blosclz'):
     return packed_array
 
 
-def unpack_array(packed_array):
+def unpack_array(packed_array, **kwargs):
     """unpack_array(packed_array)
 
     Unpack (decompress) a packed NumPy array.
@@ -622,6 +715,10 @@ def unpack_array(packed_array):
     packed_array : str / bytes
         The packed array to be decompressed.
 
+    **kwargs : fix_imports / encoding / errors
+        Optional parameters that can be passed to the pickle.loads API
+        https://docs.python.org/3/library/pickle.html#pickle.loads
+
     Returns
     -------
     out : ndarray
@@ -643,7 +740,11 @@ def unpack_array(packed_array):
     >>> a2 = blosc.unpack_array(parray)
     >>> numpy.alltrue(a == a2)
     True
-
+    >>> a = numpy.array(['å', 'ç', 'ø'])
+    >>> parray = blosc.pack_array(a)
+    >>> a2 = blosc.unpack_array(parray)
+    >>> numpy.alltrue(a == a2)
+    True
     """
 
     _check_bytesobj(packed_array)
@@ -651,7 +752,13 @@ def unpack_array(packed_array):
     # First decompress the pickle
     pickled_array = _ext.decompress(packed_array, False)
     # ... and unpickle
-    array = pickle.loads(pickled_array)
+
+    if kwargs and PY3X:
+        array = pickle.loads(pickled_array, **kwargs)
+        if all(isinstance(x, bytes) for x in array.tolist()):
+            array = np.array([x.decode('utf-8') for x in array.tolist()])
+    else:
+        array = pickle.loads(pickled_array)
 
     return array
 
